#!/usr/bin/perl
#
# buildcollections - small perl script to convert GNU Classpath's
#                    Collection classes into its own package for java 1.1
#
# Copyright (C) 2000 Jochen Hoenicke.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

my $classpath="../reposit/classpath";
my $destPkg="org.gnu.java.util.collections";
my @javalangclasses=qw(UnsupportedOperationException
		       Comparable);
my @javautilclasses=qw(AbstractCollection
		       AbstractList
		       AbstractMap
		       AbstractSequentialList
		       AbstractSet
		       ArrayList
		       Arrays
		       BasicMapEntry
		       Bucket
		       List
		       Collection
		       Collections
		       Comparator
		       ConcurrentModificationException
		       HashMap
		       HashSet
		       Hashtable
		       Iterator
		       LinkedList
		       ListIterator
		       Map
		       NoSuchElementException
		       Random
		       Set
		       SortedMap
		       SortedSet
		       TreeMap
		       TreeSet
		       Vector);

my $destpath = $destPkg;
$destpath =~ s/\./\//g;

my %imports = ( "Collections" => [ "Enumeration" ],
		"Hashtable" => [ "Dictionary", "Enumeration" ],
		"Vector" => [ "Enumeration" ]);


sub convert($$$) {
    my ($file, $infile, $outfile) = @_;

    open INPUT, "<$infile";
    open OUTPUT, ">$outfile";
    print OUTPUT <<'EOF';
/* This file was converted from the GNU Classpath Project by a
 * perl script, written by Jochen Hoenicke <jochen@gnu.org>.
 *
 * Error reports should be mailed to <jochen@gnu.org> and will get
 * back-propagated into the GNU Classpath Project.
 */
EOF
    while (<INPUT>) {
	$_ = "package $destPkg;\n" if ($_ =~ /^package java.(lang|util);$/);

	next if $_ =~ /^import java.io.Object.*putStream.*Field;$/;
	next if $_ =~ /^import java.io.ObjectStreamField;$/;

	for $clazz (@javalangclasses) {
	    $_ =~ s/java.lang.$clazz/$clazz/g;
	}
	for $clazz (@javautilclasses) {
	    $_ =~ s/java.util.$clazz/$clazz/g;
	}

	$_ =~ s/abstract (interface)/$1/g;

	if ($file eq "ArrayList") {
	    # ArrayList makes heavy use of PutField/GetField.
	    # change it.
	    $_ =~ s/_iSize/size/g;
	    $_ =~ s/_arData/data/g;
	}
	if ($file eq "TreeMap") {
	    $_ =~ s/_oComparator/comparator/g;
	    $_ =~ s/(RBNode _oRoot|int _iSize|int _iModCount)/transient $1/;
	}
	if ($file eq "Vector") {
	    $_ =~ s/(\s+)return (\w+)\.delete\((.*), x\)\./$1$2.setLength($3);\n${1}return $2./g;
	}

	if ($file eq "ArrayList"
	    || $file eq "TreeMap"
	    || $file eq "Hashtable") {
	    if ($_ =~ /serialPersistentFields/) {
		$_ = <INPUT> while !($_ =~ /\}/);
		next;
	    }

	    $_ =~ /^(\s+)(.*\W)?(\w+).readFields/ and
		$_ = "$1$3.defaultReadObject();\n";
	    $_ =~ /^(\s+)(.*\W)?(\w+).writeFields/ and
		$_ = "$1$3.defaultWriteObject();\n";

	    if ($_ =~ /[io]Fields/) {
		$_ = <INPUT> while !($_ =~ /;/);
		next;
	    }
	}

	print OUTPUT $_;
	if ($_ =~ /^package $destPkg;$/
	    && exists $imports{$file}) {
	    for $imp (@{$imports{$file}}) {
		print OUTPUT "import java.util.$imp;\n";
	    }
	}
    }
}

my $file;

for $file (@javalangclasses) {
    my $infile = "$classpath/java/lang/$file.java";
    my $outfile = "$destpath/$file.java";
    print "$outfile\n";
    convert ($file, $infile, $outfile);
}

for $file (@javautilclasses) {
    my $infile = "$classpath/java/util/$file.java";
    my $outfile = "$destpath/$file.java";
    print "$outfile\n";
    convert ($file, $infile, $outfile);
}
